#!/usr/bin/env python

import re

try:
    from urllib2 import urlparse
except:
    from urllib.parse import urlparse

# https://gist.github.com/dperini/729294
RE_URLS = re.compile(
    r'((?:(?P<protocol>https?|ftp)://)?'
    r'(?:\S+(?::\S*)?@)?'
    r'((?P<hostname>'
    r'(?!(?:10|127)(?:\.\d{1,3}){3})'
    r'(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})'
    r'(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})'
    r'(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])'
    r'(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}'
    r'(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))'
    r'|'
    r'(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)'
    r'(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*'
    r')(?P<tld>\.(?:[a-z\u00a1-\uffff]{2,}))'
    r'))'
    r'(?::\d{2,5})?'
    r'(?:/\S*)?',
    re.IGNORECASE
)

def defang(line):
    clean_line = line
    for match in RE_URLS.finditer(line):
        clean = ''
        if match.group('protocol'):
            clean += match.group('protocol').replace('t', 'X')
            clean += '://'
        clean += match.group('hostname')
        clean += match.group('tld').replace('.', '[.]')
        clean_line = clean_line.replace(match.group(1), clean)
    return clean_line

def defanger(infile, outfile):
    for line in infile:
        clean_line = defang(line)
        outfile.write(clean_line)

def refang(line):
    dirty_line = re.sub(r'\[((\.)|((dot)|DOT))\]', '.', line)
    if re.match(r'^h([x]{2}|[X]{2})p(s)*:', dirty_line):
        dirty_line = re.sub(r'^h([x]{2}|[X]{2})p', 'http', dirty_line)
    if re.match(r'^f([x]{1}|[X]{1})p:', dirty_line):
        dirty_line = re.sub(r'^f([x]{1}|[X]{1})p', 'ftp', dirty_line)
    return dirty_line

def refanger(infile, outfile):
    for line in infile:
        dirty_line = refang(line)
        outfile.write(dirty_line)



#!/usr/bin/env python
import sys
from defang import defanger, refanger

def defang():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('--refang', '-r', action='store_true', help='refangs urls')
    parser.add_argument('-i', '--input', help='input file, default stdin')
    parser.add_argument('-o', '--output', help='output file, default stdout')
    args = parser.parse_args()
    
    try:
        if args.input:
            input_f = open(args.input)
        else:
            input_f = sys.stdin
        if args.output:
            output_f = open(args.output, 'w')
        else:
            output_f = sys.stdout
        if args.refang:
            refanger(input_f, output_f)
        else:
            defanger(input_f, output_f)
    finally:
        try:
            if args.output:
                output_f.close()
        finally:
            if args.input:
                input_f.close()

if __name__ == '__main__':
    defang()



import os
from setuptools import setup

# defang
# Defangs and refangs malicious URLs

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "defang",
    version = "0.2.4",
    description = "Defangs and refangs malicious URLs",
    author = "Johan Nestaas",
    author_email = "johannestaas@gmail.com",
    license = "GPLv3+",
    keywords = "",
    url = "https://www.bitbucket.org/johannestaas/defang",
    packages=['defang'],
    package_dir={'defang': 'defang'},
    long_description=read('README.md'),
    classifiers=[
        #'Development Status :: 1 - Planning',
        #'Development Status :: 2 - Pre-Alpha',
        'Development Status :: 3 - Alpha',
        #'Development Status :: 4 - Beta',
        #'Development Status :: 5 - Production/Stable',
        #'Development Status :: 6 - Mature',
        #'Development Status :: 7 - Inactive',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Environment :: Console',
        'Environment :: X11 Applications :: Qt',
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
    ],
    install_requires=[
    ],
    entry_points = {
        'console_scripts': [
            'defang = defang.bin:defang',
        ],
    },
    #package_data = {
        #'defang': ['catalog/*.edb'],
    #},
    #include_package_data = True,
)
